%{

#include <stdio.h>
#include <stdlib.h>

unsigned int line_no = 1 ;
unsigned int col_no = 0 ;

unsigned int indent_stack[32] ; /* Ought to be enough, right? ;) */
unsigned int* stack = indent_stack ;

void change_indent(unsigned int width) {

  /* Unchanged: */
  if (width == *stack) 
    return ;
  
  /* New indent: */
  if (width > *stack) {
    stack++ ;
    *stack = width ;
    printf("{\n") ;
    return ;
  } 

  /* Unindented: */
  while (width < *stack) {
    stack-- ;
    printf("}\n") ;
  }

  if (width > *stack) {
    stack++ ;
    *stack = width ;
  }
}

%}


nl \n|\r|\r\n

ws [ \t]

id [A-Za-z_][A-Za-z_0-9]*

nzd      [1-9]
digit    [0-9]
decint   {nzd}{digit}*|0+

op    "+"
delim ","
punct {op}|{delim}

open  "("
close ")"


%%
<INITIAL>^{ws}*{nl}   {change_indent(0);}

<INITIAL>^[ ]+        {change_indent(strlen(yytext));}

<INITIAL>{nl}         {printf("\n");}
<INITIAL>def          {printf("function ");}
<INITIAL>return       {printf("return ");}
<INITIAL>{id}         {printf("%s ",yytext);}
<INITIAL>:            {}
<INITIAL>{punct}      {printf("%s",yytext);} 
<INITIAL>{close}      {printf("%s",yytext);}
<INITIAL>{open}       {printf("%s",yytext);}
<INITIAL>{decint}     {printf(" %s ",yytext);} 

<INITIAL>{ws}         {}

 /* Skip comments: */
<INITIAL>[#][^\r\n]* {}

.                     {printf("// unknown char: '%s'\n",yytext);}
%%

int yywrap(void) {
  /* Called at EOF. */
  change_indent(0) ;
  return 1 ;
}


int main(int argc, char* argv[]) {

  (void)argc; (void)argv ;
  (void)yyunput;

  indent_stack[0] = 0 ;

  yylex() ; 

  return 0 ;
}
